/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

package com.sunlabs.archsim.fleet.ships.stride;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

public class StrideTestSink extends Component {

	// terminal names, <output, input > pairs
	public static final String ACK_COUNTER= "ackCount";
	public static final String COUNTER= "count";
	
	public static final String NEXT= "next";
	public static final String ACK_NEXT= "ackNext";

	// StrideSourceAction
	Action strideSinkAction;
	
	// the terminals
	private Terminal ackCountTerm;
	private Terminal countTerm;
	private Terminal nextTerm;
	private Terminal ackNextTerm;
	
	private Map<String, Terminal> terminalNameMap;

	// remember all the counts
	public LinkedList<Data> counts;
	
	public StrideTestSink() {
		super("StrideTestSink", null);
		buildStrideTestSink();
	}
	
	public StrideTestSink(String name, Parameters params) {
		super(name, null);
		
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
		
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
		
			} // for
		} // if params != null		
		
		buildStrideTestSink();
		
	} // constructor
	
	protected void buildStrideTestSink() {
		terminalNameMap= new HashMap<String, Terminal>();

		// remember the values
		counts= new LinkedList<Data>();

		// make the action
		strideSinkAction= new StrideTestSinkAction();
		addMember(strideSinkAction);
		
		// make the terminals, form the map
		ackCountTerm= new OutputTerminal(ACK_COUNTER, this);
		terminalNameMap.put(ACK_COUNTER, ackCountTerm);
		
		countTerm= new InputTerminal(COUNTER, this);
		terminalNameMap.put(COUNTER, countTerm);
		
		nextTerm= new OutputTerminal(NEXT, this);
		terminalNameMap.put(NEXT, nextTerm);
		
		ackNextTerm= new InputTerminal(ACK_NEXT, this);
		terminalNameMap.put(ACK_NEXT, ackNextTerm);
		
		// attach the terminals to the action
		strideSinkAction.attach(ackCountTerm);
		strideSinkAction.attach(countTerm);
		strideSinkAction.attach(nextTerm);
		strideSinkAction.attach(ackNextTerm);		
	}
	
	public Terminal getTerminal(String name) {
		return terminalNameMap.get(name);
	}

	public List<Data> getResults() {
		return counts;
	}
	
	private class StrideTestSinkAction extends Action {


		
		public StrideTestSinkAction() {
			super("StrideTestSinkAction");
			
			outputFunctions.put(NEXT, new AckCountFn());
			outputFunctions.put(ACK_COUNTER, new AckFunction());
			
		} // StrideSourceAction constructor
		
		/* provide values for stride sources */
		private class AckCountFn extends Function {
			
			
			public AckCountFn() {
				super("AckCountFn");				
				// determine function delay
				Integer fDelay= globals.intValue(StrideDefaults.STRIDE_ACK_DELAY);
				functionDelay= (fDelay != null)
						? new Delay(fDelay.intValue())
						: StrideDefaults.STRIDE_ACK_DELAY_DEF;
			}
			
			public Object value() {
				Data d= (Data)countTerm.getData();
				// record the result
				counts.addLast(d);
				return new Data();
			} // value
			
		} // AckCountFn	
		
	} // class StrideTestSinkAction
	
} // StrideTestSink
